==> plot_playground-0.0.5-py3-none-any.whl: plot_playground/common/d3_helper.py <==
"""
A D3.js common helper module.

Notes
-----
Js string is using bracket, that conflict python string format.
Therefore, r character is required before quotation.
"""

import os
from datetime import datetime
from random import randint
import json
import re

from IPython.display import display, HTML

from plot_playground.common import settings
from plot_playground.common import data_helper


D3_REQUIRE_HTML = r"""
<script>
    requirejs.config({
        paths: {
            'd3': ['https://d3js.org/d3.v{d3_version}.min'],
        },
    });
</script>
""".replace(r'{d3_version}', settings.D3_VERSION)


def load_d3_on_jupyter():
    """
    Load D3.js script on Jupyter, so that it enable to
    access d3 methods.
    """
    display(HTML(D3_REQUIRE_HTML))


D3_SCRIPT_EXEC_HTML = r"""
<style>
    {css_str}
</style>

<svg id="{svg_id}" width="{svg_width}" height="{svg_height}">
</svg>
<script>
    require(['d3'], function(d3) {
        {js_script}
    });
</script>
"""


def exec_d3_js_script_on_jupyter(
        js_script, css_str, svg_id, svg_width, svg_height):
    """
    Execute the JavaScript code in a form that can access
    D3.js.

    Parameters
    ----------
    js_script : str
        Code of JavaScript to be executed.
    css_str : str
        The CSS string to set.
    svg_id : str
        ID set to SVG. Using this ID as a selector, you can
        access with jQuery or D3.js code.
    svg_width : int
        Width set to SVG in pixels.
    svg_height : int
        Height set to SVG in pixels.
    """
    load_d3_on_jupyter()
    html = D3_SCRIPT_EXEC_HTML.replace(
        r'{svg_id}', str(svg_id))
    html = html.replace(
        r'{svg_width}', str(svg_width))
    html = html.replace(
        r'{svg_height}', str(svg_height))
    html = html.replace(
        r'{js_script}', js_script)
    html = html.replace(
        r'{css_str}', css_str)
    display(HTML(html))


def make_svg_id():
    """
    Generate unique SVG ID using random number and time stamp.

    Returns
    -------
    svg_id : str
        Generated SVG ID.
    """
    timestamp_str = str(datetime.now().timestamp())
    timestamp_str = timestamp_str.replace('.', '_')
    random_int_str = str(randint(10000, 99999))
    svg_id = 'svg_id_{timestamp_str}_{random_int_str}'.format(
        timestamp_str=timestamp_str,
        random_int_str=random_int_str
    )
    return svg_id


def read_template_str(template_file_path):
    """
    Read string of template file.

    Parameters
    ----------
    template_file_path : str
        The path of the template file under the template directory.
        e.g., storytelling/simple_line_date_series_plot.css

    Returns
    -------
    template_str
        Loaded template string. The repr function is set.

    Raises
    ------
    Exception
        If the file can not be found.
    """
    file_path = os.path.join(
        settings.ROOT_DIR, 'plot_playground', 'template',
        template_file_path)
    if not os.path.exists(file_path):
        err_msg = 'Template file not found : %s' % file_path
        raise Exception(err_msg)
    with open(file_path, 'r') as f:
        template_str = f.read()
    template_str = re.sub(re.compile('/\*.*?\*/', re.DOTALL) , '', template_str)
    template_str = repr(template_str)[1:-1]
    template_str = template_str.replace('\\n', '\n')
    return template_str


def apply_css_param_to_template(css_template_str, css_param):
    """
    Apply the parameters to the CSS template.

    Parameters
    ----------
    css_template_str : str
        String of CSS template.
    css_param : dict
        A dictionary that stores parameter name in key and parameter
        in value. Parameter name corresponds to string excluding hyphens
        in template.

    Returns
    -------
    css_template_str : str
        Template string after parameters are reflected.
    """
    for key, value in css_param.items():
        key = '--%s--' % key
        css_template_str = css_template_str.replace(key, str(value))
    return css_template_str


def apply_js_param_to_template(js_template_str, js_param):
    """
    Apply the parameters to the js template.

    Parameters
    ----------
    js_template_str : str
        String of js template.
    js_param : dict
        A dictionary that stores parameter name in key and parameter
        in value. If the parameter is a list or dictionary, it is
        converted to Json format.

    Returns
    -------
    js_template_str : str
        Template string after parameters are reflected.
    """
    for key, value in js_param.items():
        if isinstance(value, (dict, list)):
            value = data_helper.convert_dict_or_list_numpy_val_to_python_val(
                target_obj=value)
            value = json.dumps(value)
        key = r'{' + key + r'}'
        value = str(value)
        js_template_str = js_template_str.replace(key, value)
    return js_template_str


class PlotMeta():

    def __init__(
            self, js_template_str, js_param, css_template_str,
            css_param):
        """
        Class dealing with plot metadata.

        Parameters
        ----------
        js_template_str : str
            JavaScript template string after parameter substitution.
        js_param : dict
            A dictionary storing parameters set in the JavaScript template.
        css_template_str : str
            CSS template string after parameter substitution.
        css_param : dict
            A dictionary storing parameters set in the CSS template.
        """
        self.js_template_str = js_template_str
        self.js_param = js_param
        self.css_template_str = css_template_str
        self.css_param = css_param

==> plot_playground-0.0.8-py3-none-any.whl: plot_playground/common/d3_helper.py <==
"""
A D3.js common helper module.

Notes
-----
Js string is using bracket, that conflict python string format.
Therefore, r character is required before quotation.
"""

import os
from datetime import datetime
from random import randint
import json
import re

from IPython.display import display, HTML

from plot_playground.common import settings
from plot_playground.common import data_helper


def read_template_str(template_file_path):
    """
    Read string of template file.

    Parameters
    ----------
    template_file_path : str
        The path of the template file under the template directory.
        e.g., storytelling/simple_line_date_series_plot.css

    Returns
    -------
    template_str
        Loaded template string. The repr function is set.

    Raises
    ------
    Exception
        If the file can not be found.
    """
    file_path = os.path.join(
        settings.ROOT_DIR, 'plot_playground', 'template',
        template_file_path)
    if not os.path.exists(file_path):
        err_msg = 'Template file not found : %s' % file_path
        raise Exception(err_msg)
    with open(file_path, 'r') as f:
        template_str = f.read()
    template_str = re.sub(re.compile('/\*.*?\*/', re.DOTALL) , '', template_str)
    template_str = repr(template_str)[1:-1]
    template_str = template_str.replace('\\n', '\n')
    template_str = template_str.replace('\\\\', '\\')
    return template_str


def apply_css_param_to_template(css_template_str, css_param):
    """
    Apply the parameters to the CSS template.

    Parameters
    ----------
    css_template_str : str
        String of CSS template.
    css_param : dict
        A dictionary that stores parameter name in key and parameter
        in value. Parameter name corresponds to string excluding hyphens
        in template.

    Returns
    -------
    css_template_str : str
        Template string after parameters are reflected.
    """
    for key, value in css_param.items():
        key = '--%s--' % key
        css_template_str = css_template_str.replace(key, str(value))
    return css_template_str


def apply_js_param_to_template(js_template_str, js_param):
    """
    Apply the parameters to the js template.

    Parameters
    ----------
    js_template_str : str
        String of js template.
    js_param : dict
        A dictionary that stores parameter name in key and parameter
        in value. If the parameter is a list or dictionary, it is
        converted to Json format.

    Returns
    -------
    js_template_str : str
        Template string after parameters are reflected.
    """
    for key, value in js_param.items():
        if isinstance(value, (dict, list)):
            value = data_helper.convert_dict_or_list_numpy_val_to_python_val(
                target_obj=value)
            value = json.dumps(value)
        key = r'{' + key + r'}'
        value = str(value)
        js_template_str = js_template_str.replace(key, value)
    return js_template_str


def _read_d3_require_html():
    """
    Read the template for loading D3.js.

    Returns
    -------
    d3_require_html : str
        The template string for loading D3.js.
    """
    d3_require_html = read_template_str(
        template_file_path='base/d3_load.html')
    param_dict = {
        'd3_version': settings.D3_VERSION,
    }
    d3_require_html = apply_js_param_to_template(
        js_template_str=d3_require_html,
        js_param=param_dict)
    return d3_require_html


D3_REQUIRE_HTML = _read_d3_require_html()


def load_d3_on_jupyter():
    """
    Load D3.js script on Jupyter, so that it enable to
    access d3 methods.
    """
    display(HTML(D3_REQUIRE_HTML))


D3_SCRIPT_EXEC_HTML = read_template_str(
    template_file_path='base/d3_exec.html')


def exec_d3_js_script_on_jupyter(
        js_script, css_str, svg_id, svg_width, svg_height):
    """
    Execute the JavaScript code in a form that can access
    D3.js.

    Parameters
    ----------
    js_script : str
        Code of JavaScript to be executed.
    css_str : str
        The CSS string to set.
    svg_id : str
        ID set to SVG. Using this ID as a selector, you can
        access with jQuery or D3.js code.
    svg_width : int
        Width set to SVG in pixels.
    svg_height : int
        Height set to SVG in pixels.
    """
    load_d3_on_jupyter()
    html = D3_SCRIPT_EXEC_HTML.replace(
        r'{svg_id}', str(svg_id))
    html = html.replace(
        r'{svg_width}', str(svg_width))
    html = html.replace(
        r'{svg_height}', str(svg_height))
    html = html.replace(
        r'{js_script}', js_script)
    html = html.replace(
        r'{css_str}', css_str)
    display(HTML(html))


def make_svg_id():
    """
    Generate unique SVG ID using random number and time stamp.

    Returns
    -------
    svg_id : str
        Generated SVG ID.
    """
    timestamp_str = str(datetime.now().timestamp())
    timestamp_str = timestamp_str.replace('.', '_')
    random_int_str = str(randint(10000, 99999))
    svg_id = 'svg_id_{timestamp_str}_{random_int_str}'.format(
        timestamp_str=timestamp_str,
        random_int_str=random_int_str
    )
    return svg_id


class PlotMeta():

    def __init__(
            self, js_template_str, js_param, css_template_str,
            css_param):
        """
        Class dealing with plot metadata.

        Parameters
        ----------
        js_template_str : str
            JavaScript template string after parameter substitution.
        js_param : dict
            A dictionary storing parameters set in the JavaScript template.
        css_template_str : str
            CSS template string after parameter substitution.
        css_param : dict
            A dictionary storing parameters set in the CSS template.
        """
        self.js_template_str = js_template_str
        self.js_param = js_param
        self.css_template_str = css_template_str
        self.css_param = css_param
